@@ -1,5 +1,5 @@
 from constants import *
-from database import Database
+from cache import persist_db
 from config import Config
 import os
 import logging
@@ -15,7 +15,7 @@ logger = None
 
 class AppServer(Config):
     bool_options = {'debug', 'log_debug', 'auto_start_servers',
-        'get_public_ip', 'inline_certs'}
+        'get_public_ip', 'inline_certs', 'ssl'}
     int_options = {'port', 'session_timeout', 'key_bits', 'dh_param_bits'}
     path_options = {'log_path', 'db_path', 'www_path', 'data_path',
         'server_cert_path', 'server_key_path'}
@@ -24,6 +24,7 @@ class AppServer(Config):
         'auto_start_servers': True,
         'get_public_ip': True,
         'inline_certs': True,
+        'ssl': True,
         'session_timeout': DEFAULT_SESSION_TIMEOUT,
         'key_bits': DEFAULT_KEY_BITS,
         'dh_param_bits': DEFAULT_DH_PARAM_BITS,
@@ -36,10 +37,15 @@ class AppServer(Config):
     def __init__(self):
         Config.__init__(self)
         self.app = None
-        self.app_db = None
-        self.mem_db = None
         self.interrupt = False
 
+    def __getattr__(self, name):
+        if name == 'web_protocol':
+            if self.debug or not self.ssl:
+                return 'http'
+            return 'https'
+        return Config.__getattr__(self, name)
+
     def load_public_ip(self, retry=False, timeout=3):
         if not self.get_public_ip:
             return
@@ -66,6 +72,16 @@ class AppServer(Config):
         self.app = flask.Flask(APP_NAME)
         self.app.secret_key = os.urandom(32)
 
+        @self.app.before_request
+        def before_request():
+          flask.g.start = time.time()
+
+        @self.app.after_request
+        def after_request(response):
+            response.headers.add('Execution-Time',
+                int((time.time() - flask.g.start) * 1000))
+            return response
+
         global logger
         logger = self.app.logger
 
@@ -73,12 +89,11 @@ class AppServer(Config):
         import flask
         from auth_token import AuthToken
         def _wrapped(*args, **kwargs):
-            if flask.request.json and 'auth_token' in flask.request.json and \
-                    flask.request.json['auth_token']:
-                auth_token = AuthToken(flask.request.json['auth_token'])
+            auth_token = flask.request.headers.get('Auth-Token', None)
+            if auth_token:
+                auth_token = AuthToken(auth_token)
                 if not auth_token.valid:
                     raise flask.abort(401)
-
             else:
                 if 'timestamp' not in flask.session:
                     raise flask.abort(401)
@@ -92,6 +107,15 @@ class AppServer(Config):
         _wrapped.__name__ = '%s_auth' % call.__name__
         return _wrapped
 
+    def local_only(self, call):
+        import flask
+        def _wrapped(*args, **kwargs):
+            if flask.request.remote_addr != '127.0.0.1':
+                raise flask.abort(401)
+            return call(*args, **kwargs)
+        _wrapped.__name__ = '%s_local_only' % call.__name__
+        return _wrapped
+
     def _setup_conf(self):
         self.set_path(self.conf_path)
         if not os.path.isdir(self.data_path):
@@ -122,14 +146,7 @@ class AppServer(Config):
         logger.addHandler(self.log_handler)
 
     def _setup_db(self):
-        self.mem_db = Database(None)
-        self.app_db = Database(self.db_path)
-
-    def _close_db(self):
-        if self.mem_db:
-            self.mem_db.close()
-        if self.app_db:
-            self.app_db.close()
+        persist_db.setup_persist(self.db_path)
 
     def _setup_handlers(self):
         import handlers
@@ -149,34 +166,58 @@ class AppServer(Config):
     def _get_version_int(self, version):
         return int(''.join([x.zfill(2) for x in version.split('.')]))
 
-    def _upgrade_data(self):
-        from pritunl import __version__
-        version = self._get_version_int(__version__)
-        cur_version = '0.10.1'
+    def _get_data_version(self):
         version_path = os.path.join(self.data_path, VERSION_NAME)
         if os.path.isfile(version_path):
             with open(version_path, 'r') as version_file:
-                cur_version = version_file.readlines()[0].strip()
-        cur_version = self._get_version_int(cur_version)
+                return self._get_version_int(
+                    version_file.readlines()[0].strip())
 
-        if cur_version < self._get_version_int('0.10.2'):
-            logger.info('Upgrading data to v0.10.2...')
-            from organization import Organization
-            for org in Organization.get_orgs():
-                for user in org.get_users():
-                    user._upgrade_0_10_2()
+    def _upgrade_db(self):
+        from pritunl import __version__
+        version = self._get_version_int(__version__)
+        cur_version = self._get_data_version()
 
-        if cur_version < self._get_version_int('0.10.4'):
+        if cur_version and cur_version < self._get_version_int('0.10.5'):
+            logger.info('Upgrading database to v0.10.5...')
+            try:
+                os.remove(self.db_path)
+            except OSError:
+                pass
+
+    def _upgrade_data(self):
+        from pritunl import __version__
+        version = self._get_version_int(__version__)
+        cur_version = self._get_data_version()
+
+        if cur_version and cur_version < self._get_version_int('0.10.4'):
             logger.info('Upgrading data to v0.10.4...')
             from organization import Organization
-            for org in Organization.get_orgs():
-                for user in org.get_users():
+            for org in Organization.iter_orgs():
+                for user in org.iter_users():
                     user._upgrade_0_10_4()
 
+        if cur_version and cur_version < self._get_version_int('0.10.5'):
+            logger.info('Upgrading data to v0.10.5...')
+            from server import Server
+            for server in Server.iter_servers():
+                server._upgrade_0_10_5()
+
+            from organization import Organization
+            for org in Organization.iter_orgs():
+                org._upgrade_0_10_5()
+
         if cur_version != version:
+            version_path = os.path.join(self.data_path, VERSION_NAME)
             with open(version_path, 'w') as version_file:
                 version_file.write('%s\n' % __version__)
 
+    def _fill_cache(self):
+        logger.info('Preloading cache...')
+        from organization import Organization
+        for org in Organization.iter_orgs():
+            org._cache_users()
+
     def _hash_password(self, password):
         password_hash = hashlib.sha512()
         password_hash.update(password)
@@ -203,19 +244,22 @@ class AppServer(Config):
         self._setup_conf()
         self._setup_log()
         self._setup_public_ip()
+        self._upgrade_db()
         self._setup_db()
         self._setup_handlers()
         self._setup_static_handler()
         self._upgrade_data()
+        self._fill_cache()
 
     def _setup_server_cert(self):
         if self.server_cert_path and self.server_key_path:
             self._server_cert_path = self.server_cert_path
             self._server_key_path = self.server_key_path
         else:
-            data_path = self.data_path or DEFAULT_DATA_PATH
-            self._server_cert_path = os.path.join(data_path, SERVER_CERT_NAME)
-            self._server_key_path = os.path.join(data_path, SERVER_KEY_NAME)
+            self._server_cert_path = os.path.join(self.data_path,
+                SERVER_CERT_NAME)
+            self._server_key_path = os.path.join(self.data_path,
+                SERVER_KEY_NAME)
 
             if not os.path.isfile(self._server_cert_path) or \
                     not os.path.isfile(self._server_key_path):
@@ -234,7 +278,8 @@ class AppServer(Config):
                 os.chmod(self._server_key_path, 0600)
 
     def _run_wsgi(self):
-        self._setup_server_cert()
+        if self.ssl:
+            self._setup_server_cert()
         import cherrypy.wsgiserver
         import cherrypy.wsgiserver.ssl_builtin
         from log_entry import LogEntry
@@ -242,15 +287,23 @@ class AppServer(Config):
 
         if self.auto_start_servers:
             from pritunl.server import Server
-            for server in Server.get_servers():
-                if server.get_orgs():
-                    server.start()
+            for server in Server.iter_servers():
+                if server.org_count:
+                    try:
+                        server.start()
+                    except:
+                        logger.exception('Failed to auto start server. %r' % {
+                            'server_id': server.id,
+                        })
 
         server = cherrypy.wsgiserver.CherryPyWSGIServer(
             (self.bind_addr, self.port), self.app,
+            request_queue_size=SERVER_REQUEST_QUEUE_SIZE,
             server_name=cherrypy.wsgiserver.CherryPyWSGIServer.version)
-        server.ssl_adapter = cherrypy.wsgiserver.ssl_builtin.BuiltinSSLAdapter(
-            self._server_cert_path, self._server_key_path)
+        if self.ssl:
+            server.ssl_adapter = \
+                cherrypy.wsgiserver.ssl_builtin.BuiltinSSLAdapter(
+                    self._server_cert_path, self._server_key_path)
         try:
             server.start()
         except (KeyboardInterrupt, SystemExit):
@@ -260,11 +313,10 @@ class AppServer(Config):
             raise
         finally:
             signal.signal(signal.SIGINT, signal.SIG_IGN)
-            LogEntry(message='Web server stopped.')
             self.interrupt = True
             logger.info('Stopping server...')
             server.stop()
-            self._close_db()
+            LogEntry(message='Web server stopped.')
 
     def _run_wsgi_debug(self):
         from log_entry import LogEntry
@@ -284,11 +336,9 @@ class AppServer(Config):
             raise
         finally:
             signal.signal(signal.SIGINT, signal.SIG_IGN)
-            LogEntry(message='Web server stopped.')
             self.interrupt = True
-            # Possible data loss here db closing before debug server
-            self._close_db()
             logger.info('Stopping debug server...')
+            LogEntry(message='Web server stopped.')
 
     def _run_server(self):
         from log_entry import LogEntry
@@ -299,8 +349,5 @@ class AppServer(Config):
             self._run_wsgi()
 
     def run_server(self):
-        try:
-            self._setup_all()
-            self._run_server()
-        finally:
-            self._close_db()
+        self._setup_all()
+        self._run_server()
