@@ -1,11 +1,9 @@
 from constants import *
+from cache import cache_db
 import os
 import logging
 
 logger = logging.getLogger(APP_NAME)
-_RESERVED_ATTRIBUTES = {'bool_options', 'int_options', 'float_options',
-    'path_options', 'str_options', 'list_options', 'default_options',
-    'all_options'}
 
 class Config:
     bool_options = set()
@@ -16,6 +14,8 @@ class Config:
     list_options = set()
     default_options = {}
     chmod_mode = None
+    cached = False
+    cache_prefix = None
 
     def __init__(self, path=None):
         self.all_options = self.bool_options | self.int_options | \
@@ -25,22 +25,21 @@ class Config:
         self.set_state(SAVED)
 
     def __setattr__(self, name, value):
-        if name in _RESERVED_ATTRIBUTES:
-            pass
-        elif name in self.all_options:
+        if name != 'all_options' and name in self.all_options:
             self.set_state(UNSAVED)
         self.__dict__[name] = value
 
     def __getattr__(self, name):
-        if name in _RESERVED_ATTRIBUTES:
-            pass
-        elif name in self.all_options:
+        if name in self.all_options:
             if not self._loaded:
                 self.load()
             if name not in self.__dict__:
                 if name in self.default_options:
                     return self.default_options[name]
-                return [] if name in self.list_options else None
+                if name in self.list_options:
+                    self.__dict__[name] = []
+                else:
+                    return
         elif name not in self.__dict__:
             raise AttributeError('Config instance has no attribute %r' % name)
         return self.__dict__[name]
@@ -138,10 +137,39 @@ class Config:
 
         return name, value
 
+    def get_cache_key(self, suffix=None):
+        if not self.cache_prefix:
+            raise AttributeError('Cached config object requires cache_prefix')
+        key = '%s-%s' % (self.cache_prefix, self.id)
+        if suffix:
+            key += '-%s' % suffix
+        return key
+
+    def clear_cache(self):
+        cache_db.remove(self.get_cache_key('cached'))
+        cache_db.remove(self.get_cache_key())
+
     def load(self, merge=False):
-        logger.debug('Loading config.')
+        logger.debug('Loading config. %r' % {
+            'path': self._conf_path,
+        })
         self._loaded = True
 
+        if self.cached:
+            if not hasattr(self, 'id'):
+                raise ValueError('Object ID is required for caching')
+            if cache_db.get(self.get_cache_key('cached')) == 't':
+                if merge:
+                    for name, value in cache_db.dict_get_all(
+                            self.get_cache_key()).iteritems():
+                        if name in self.__dict__:
+                            continue
+                        self.__dict__[name] = value
+                else:
+                    self.__dict__.update(cache_db.dict_get_all(
+                        self.get_cache_key()))
+                return
+
         try:
             with open(self._conf_path) as config:
                 for line in config:
@@ -164,6 +192,9 @@ class Config:
                         if merge and name in self.__dict__:
                             continue
                         self.__dict__[name] = value
+                        if self.cached:
+                            cache_db.dict_set(self.get_cache_key(),
+                                name, value)
                     except ValueError:
                         logger.warning('Ignoring invalid line. %r' % {
                             'line': line,
@@ -172,6 +203,9 @@ class Config:
             if not merge:
                 raise
 
+        if self.cached:
+            cache_db.set(self.get_cache_key('cached'), 't')
+
     def commit(self):
         logger.debug('Committing config.')
         if not self._loaded:
@@ -189,6 +223,8 @@ class Config:
                     value = self.__dict__[name]
                     if value is None:
                         continue
+                    if self.cached:
+                        cache_db.dict_set(self.get_cache_key(), name, value)
                     config.write(self._encode_line(name, value))
             os.rename(temp_conf_path, self._conf_path)
         except:
