@@ -29,47 +29,52 @@ def export_get():
 
     tar_file = tarfile.open(data_archive_path, 'w')
     try:
+        tar_add(tar_file, os.path.join(data_path, AUTH_LOG_NAME))
+        tar_add(tar_file, os.path.join(data_path, 'pritunl.db'))
         tar_add(tar_file, os.path.join(data_path, SERVER_CERT_NAME))
         tar_add(tar_file, os.path.join(data_path, SERVER_KEY_NAME))
         tar_add(tar_file, os.path.join(data_path, VERSION_NAME))
 
-        for org in Organization.get_orgs():
-            tar_add(tar_file, org.index_path)
-            tar_add(tar_file, org.index_attr_path)
-            tar_add(tar_file, org.serial_path)
-            tar_add(tar_file, org.crl_path)
+        for org in Organization.iter_orgs():
             tar_add(tar_file, org.get_path())
             tar_file.add(empty_temp_path,
                 arcname=os.path.relpath(os.path.join(org.path, TEMP_DIR),
                     data_path))
 
-            indexed_certs_path = os.path.join(org.path, INDEXED_CERTS_DIR)
-            for path in os.listdir(indexed_certs_path):
-                tar_add(tar_file, os.path.join(indexed_certs_path, path))
-
-            for user in org.get_users() + [org.ca_cert]:
+            for user in org.iter_users():
                 tar_add(tar_file, user.reqs_path)
-                tar_add(tar_file, user.ssl_conf_path)
                 tar_add(tar_file, user.key_path)
                 tar_add(tar_file, user.cert_path)
-                tar_add(tar_file, user.key_archive_path)
                 tar_add(tar_file, user.get_path())
 
-        for server in Server.get_servers():
+            tar_add(tar_file, org.ca_cert.reqs_path)
+            tar_add(tar_file, org.ca_cert.key_path)
+            tar_add(tar_file, org.ca_cert.cert_path)
+            tar_add(tar_file, org.ca_cert.get_path())
+
+        for server in Server.iter_servers():
             tar_add(tar_file, server.dh_param_path)
             tar_add(tar_file, server.ifc_pool_path)
             tar_add(tar_file, server.get_path())
+            tar_add(tar_file, os.path.join(server.path, NODE_SERVER_NAME))
             tar_file.add(empty_temp_path,
                 arcname=os.path.relpath(os.path.join(server.path, TEMP_DIR),
                     data_path))
-    finally:
+
         tar_file.close()
 
-    with open(data_archive_path, 'r') as archive_file:
-        response = flask.Response(response=archive_file.read(),
-            mimetype='application/octet-stream')
-        response.headers.add('Content-Disposition',
-            'attachment; filename="%s"' % data_archive_name)
-
-    os.remove(data_archive_path)
-    return response
+        with open(data_archive_path, 'r') as archive_file:
+            response = flask.Response(response=archive_file.read(),
+                mimetype='application/octet-stream')
+            response.headers.add('Content-Disposition',
+                'attachment; filename="%s"' % data_archive_name)
+        return response
+    finally:
+        try:
+            tar_file.close()
+        except OSError:
+            pass
+        try:
+            os.remove(data_archive_path)
+        except OSError:
+            pass
