@@ -1,9 +1,10 @@
 from constants import *
-from pritunl import app_server, openssl_lock
+from pritunl import app_server
 from config import Config
 from organization import Organization
 from event import Event
 from log_entry import LogEntry
+from cache import cache_db
 import uuid
 import os
 import signal
@@ -13,45 +14,31 @@ import threading
 import logging
 import traceback
 import utils
+import re
+import json
 
 logger = logging.getLogger(APP_NAME)
-_threads = {}
-_events = {}
-_output = {}
-_process = {}
-_start_time = {}
 
 class Server(Config):
     str_options = {'name', 'network', 'interface', 'protocol',
-        'local_network', 'public_address', 'primary_organization',
-        'primary_user', 'organizations'}
+        'local_networks', 'public_address', 'primary_organization',
+        'primary_user', 'organizations', 'local_network'}
     bool_options = {'otp_auth', 'lzo_compression', 'debug'}
     int_options = {'port'}
-    list_options = {'organizations'}
+    list_options = {'organizations', 'local_networks'}
+    cache_prefix = 'server'
+    type = 'server'
 
-    def __init__(self, id=None, name=None, network=None, interface=None,
-            port=None, protocol=None, local_network=None, public_address=None,
-            otp_auth=None, lzo_compression=None, debug=None, organizations=[]):
+    def __init__(self, id=None, **kwargs):
         Config.__init__(self)
         self._cur_event = None
         self._last_event = 0
 
         if id is None:
-            self._initialized = False
             self.id = uuid.uuid4().hex
-            self.name = name
-            self.network = network
-            self.interface = interface
-            self.port = port
-            self.protocol = protocol
-            self.local_network = local_network
-            self.public_address = public_address
-            self.otp_auth = otp_auth
-            self.lzo_compression = lzo_compression
-            self.debug = debug
-            self.organizations = organizations
+            for name, value in kwargs.iteritems():
+                setattr(self, name, value)
         else:
-            self._initialized = True
             self.id = id
 
         self.path = os.path.join(app_server.data_path, SERVERS_DIR, self.id)
@@ -65,27 +52,77 @@ class Server(Config):
             USER_PASS_VERIFY_NAME)
         self.ovpn_status_path = os.path.join(self.path, TEMP_DIR,
             OVPN_STATUS_NAME)
+        self.auth_log_path = os.path.join(app_server.data_path, AUTH_LOG_NAME)
         self.set_path(os.path.join(self.path, 'server.conf'))
 
-        if not self._initialized:
+        if id is None:
             self._initialize()
 
+    def __setattr__(self, name, value):
+        if name == 'status':
+            if value:
+                cache_db.dict_set(self.get_cache_key(), name, 't')
+            else:
+                cache_db.dict_set(self.get_cache_key(), name, 'f')
+        elif name == 'clients':
+            cache_db.dict_set(self.get_cache_key(), name, json.dumps(value))
+        else:
+            Config.__setattr__(self, name, value)
+
     def __getattr__(self, name):
         if name == 'status':
-            if self.id in _threads:
-                return _threads[self.id].is_alive()
+            if cache_db.dict_get(self.get_cache_key(), name) == 't':
+                return True
             return False
         elif name == 'uptime':
-            if self.status and self.id in _start_time:
-                return int(time.time()) - _start_time[self.id]
+            if self.status:
+                return int(time.time()) - int(cache_db.dict_get(
+                    self.get_cache_key(), 'start_time'))
             return None
+        elif name == 'clients':
+            clients = cache_db.dict_get(self.get_cache_key(), name)
+            if self.status and clients:
+                return json.loads(clients)
+            return {}
+        elif name == 'output':
+            return '\n'.join(cache_db.list_elements(
+                self.get_cache_key('output')))
         elif name == 'user_count':
             return self._get_user_count()
         elif name == 'org_count':
             return self._get_org_count()
-
         return Config.__getattr__(self, name)
 
+    def dict(self):
+        return {
+            'id': self.id,
+            'name': self.name,
+            'type': self.type,
+            'status': self.status,
+            'uptime': self.uptime,
+            'users_online': len(self.clients),
+            'user_count': self.user_count,
+            'network': self.network,
+            'interface': self.interface,
+            'port': self.port,
+            'protocol': self.protocol,
+            'local_networks': self.local_networks,
+            'public_address': self.public_address,
+            'otp_auth': True if self.otp_auth else False,
+            'lzo_compression': self.lzo_compression,
+            'debug': True if self.debug else False,
+            'org_count': self.org_count,
+        }
+
+    def _upgrade_0_10_5(self):
+        if self.local_network:
+            logger.debug('Upgrading server to v0.10.5... %r' % {
+                'server_id': self.id,
+            })
+            self.local_networks = [self.local_network]
+            self.local_network = None
+            self.commit()
+
     def _initialize(self):
         logger.info('Initialize new server. %r' % {
             'server_id': self.id,
@@ -124,18 +161,10 @@ class Server(Config):
         logger.info('Removing server. %r' % {
             'server_id': self.id,
         })
-
         name = self.name
 
         if self.status:
-            self.force_stop()
-            for i in xrange(20):
-                if not self.status:
-                    break
-                time.sleep(0.1)
-            if self.status:
-                self.force_stop()
-                time.sleep(0.5)
+            self.force_stop(True)
 
         self._remove_primary_user()
         utils.rmtree(self.path)
@@ -147,13 +176,13 @@ class Server(Config):
         Event(type=SERVERS_UPDATED)
 
     def _create_primary_user(self):
-        if not self.get_orgs():
+        if not self.org_count:
             raise ValueError('Primary user cannot be created without ' + \
                 'any organizations')
         logger.debug('Creating primary user. %r' % {
             'server_id': self.id,
         })
-        org = self.get_orgs()[0]
+        org = self.iter_orgs().next()
         self.primary_organization = org.id
         user = org.new_user(CERT_SERVER, SERVER_USER_PREFIX + self.id)
         self.primary_user = user.id
@@ -173,7 +202,7 @@ class Server(Config):
             'server_id': self.id,
             'org_id': org_id,
         })
-        org = Organization(org_id)
+        org = Organization.get_org(id=org_id)
         if org.id in self.organizations:
             logger.debug('Organization already on server, skipping. %r' % {
                 'server_id': self.id,
@@ -198,13 +227,12 @@ class Server(Config):
         if not primary_organization or not primary_user:
             return
 
-        org = Organization(primary_organization)
+        org = Organization.get_org(id=primary_organization)
         user = org.get_user(primary_user)
         if not user:
             logger.debug('Primary user not found, skipping remove. %r' % {
                 'server_id': self.id,
                 'org_id': org.id,
-                'user_id': user.id,
             })
             return
 
@@ -225,33 +253,46 @@ class Server(Config):
         Event(type=SERVERS_UPDATED)
         Event(type=SERVER_ORGS_UPDATED, resource_id=self.id)
 
-    def get_orgs(self):
-        orgs = []
+    def iter_orgs(self):
+        orgs_dict = {}
+        orgs_sort = []
+
         for org_id in self.organizations:
-            org = Organization(org_id)
-            if not os.path.isfile(org.ca_cert.cert_path):
-                logger.warning('Removing non existent organization ' + \
-                    'from server. %r' % {
-                        'server_id': self.id,
+            org = Organization.get_org(id=org_id)
+            if not org:
+                continue
+            name_id = '%s_%s' % (org.name, org.id)
+            orgs_dict[name_id] = org
+            orgs_sort.append(name_id)
+
+        for name_id in sorted(orgs_sort):
+            yield orgs_dict[name_id]
+
+    def get_org(self, org_id):
+        for org_id in self.organizations:
+            if org_id != org_id:
+                continue
+            org = Organization.get_org(id=org_id)
+            try:
+                org.load()
+            except IOError:
+                logger.exception('Failed to load org conf. %r' % {
                         'org_id': org_id,
                     })
-                self.remove_org(org_id)
-                continue
-            orgs.append(org)
-        return orgs
+                return
+            return org
 
     def _get_user_count(self):
-        server_orgs = self.get_orgs()
         users_count = 0
-        for org in server_orgs:
-            for user in org.get_users():
-                if user.type != CERT_CLIENT:
-                    continue
-                users_count += 1
+        for org in self.iter_orgs():
+            users_count += org.user_count
         return users_count
 
     def _get_org_count(self):
-        return len(self.get_orgs())
+        org_count = 0
+        for org in self.iter_orgs():
+            org_count += 1
+        return org_count
 
     def _generate_dh_param(self):
         logger.debug('Generating server dh params. %r' % {
@@ -279,7 +320,7 @@ class Server(Config):
             'server_id': self.id,
         })
         with open(self.ca_cert_path, 'w') as server_ca_cert:
-            for org in self.get_orgs():
+            for org in self.iter_orgs():
                 ca_path = org.ca_cert.cert_path
                 with open(ca_path, 'r') as org_ca_cert:
                     server_ca_cert.write(org_ca_cert.read())
@@ -289,33 +330,31 @@ class Server(Config):
             'server_id': self.id,
         })
         with open(self.tls_verify_path, 'w') as tls_verify_file:
+            os.chmod(self.tls_verify_path, 0755)
             data_path = app_server.data_path
             tls_verify_file.write(TLS_VERIFY_SCRIPT % (
-                data_path,
-                ORGS_DIR,
-                AUTH_LOG_NAME,
-                INDEX_NAME,
+                self.auth_log_path,
+                app_server.web_protocol,
+                app_server.port,
+                self.id,
             ))
-        os.chmod(self.tls_verify_path, 0755)
 
     def _generate_user_pass_verify(self):
         logger.debug('Generating user pass verify script. %r' % {
             'server_id': self.id,
         })
         with open(self.user_pass_verify_path, 'w') as user_pass_verify_file:
+            os.chmod(self.user_pass_verify_path, 0755)
             data_path = app_server.data_path
             user_pass_verify_file.write(USER_PASS_VERIFY_SCRIPT % (
-                data_path,
-                ORGS_DIR,
-                USERS_DIR,
-                TEMP_DIR,
-                AUTH_LOG_NAME,
-                OTP_JSON_NAME,
+                self.auth_log_path,
+                app_server.web_protocol,
+                app_server.port,
+                self.id,
             ))
-        os.chmod(self.user_pass_verify_path, 0755)
 
-    def _generate_ovpn_conf(self):
-        if not self.get_orgs():
+    def _generate_ovpn_conf(self, inline=False):
+        if not self.org_count:
             raise ValueError('Ovpn conf cannot be generated without ' + \
                 'any organizations')
 
@@ -329,35 +368,51 @@ class Server(Config):
         if not os.path.isfile(self.dh_param_path):
             self._generate_dh_param()
 
-        primary_org = Organization(self.primary_organization)
+        primary_org = Organization.get_org(id=self.primary_organization)
         primary_user = primary_org.get_user(self.primary_user)
 
         self.generate_ca_cert()
         self._generate_tls_verify()
         self._generate_user_pass_verify()
 
-        if self.local_network:
-            push = 'route %s %s' % self._parse_network(
-                self.local_network)
+        if self.local_networks:
+            push = ''
+            for network in self.local_networks:
+                push += 'push "route %s %s"\n' % self._parse_network(network)
+            push = push.rstrip()
         else:
-            push = 'redirect-gateway'
+            push = 'push "redirect-gateway"'
 
-        server_conf = OVPN_SERVER_CONF % (
-            self.port,
-            self.protocol,
-            self.interface,
-            self.ca_cert_path,
-            primary_user.cert_path,
-            primary_user.key_path,
-            self.tls_verify_path,
-            self.dh_param_path,
-            '%s %s' % self._parse_network(self.network),
-            self.ifc_pool_path,
-            push,
-            self.ovpn_status_path,
-            4 if self.debug else 1,
-            8 if self.debug else 3,
-        )
+        if not inline:
+            server_conf = OVPN_SERVER_CONF % (
+                self.port,
+                self.protocol,
+                self.interface,
+                self.ca_cert_path,
+                primary_user.cert_path,
+                primary_user.key_path,
+                self.tls_verify_path,
+                self.dh_param_path,
+                '%s %s' % self._parse_network(self.network),
+                self.ifc_pool_path,
+                push,
+                self.ovpn_status_path,
+                4 if self.debug else 1,
+                8 if self.debug else 3,
+            )
+        else:
+            server_conf = OVPN_INLINE_SERVER_CONF % (
+                self.port,
+                self.protocol,
+                self.interface,
+                self.tls_verify_path,
+                '%s %s' % self._parse_network(self.network),
+                self.ifc_pool_path,
+                push,
+                self.ovpn_status_path,
+                4 if self.debug else 1,
+                8 if self.debug else 3,
+            )
 
         if self.otp_auth:
             server_conf += 'auth-user-pass-verify %s via-file\n' % (
@@ -366,10 +421,22 @@ class Server(Config):
         if self.lzo_compression:
             server_conf += 'comp-lzo\npush "comp-lzo"\n'
 
-        if self.local_network:
+        if self.local_networks:
             server_conf += 'client-to-client\n'
 
+        if inline:
+            server_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
+                self.ca_cert_path)
+            server_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
+                primary_user.cert_path)
+            server_conf += '<key>\n%s\n</key>\n' % open(
+                primary_user.key_path).read().strip()
+            server_conf += '<dh>\n%s\n</dh>\n' % open(
+                self.dh_param_path).read().strip()
+
         with open(self.ovpn_conf_path, 'w') as ovpn_conf:
+            if inline:
+                os.chmod(self.ovpn_conf_path, 0600)
             ovpn_conf.write(server_conf)
 
     def _enable_ip_forwarding(self):
@@ -382,32 +449,11 @@ class Server(Config):
             })
             raise
 
-    def _check_output(self, *popenargs, **kwargs):
-        # For python2.6 support
-        if 'stdout' in kwargs:
-            raise ValueError('stdout argument not allowed, ' + \
-                'it will be overridden.')
-        process = subprocess.Popen(stdout=subprocess.PIPE,
-            *popenargs, **kwargs)
-        output, unused_err = process.communicate()
-        retcode = process.poll()
-        if retcode:
-            cmd = kwargs.get('args')
-            if cmd is None:
-                cmd = popenargs[0]
-            raise subprocess.CalledProcessError(retcode, cmd, output=output)
-        return output
-
-    def _generate_iptable_rule(self):
-        args = []
-        match_network = '0.0.0.0'
-
-        if self.local_network:
-            match_network = self._parse_network(self.local_network)[0]
-            args += ['-d', self.local_network]
+    def _generate_iptable_rules(self):
+        iptable_rules = []
 
         try:
-            routes = self._check_output(['route', '-n'],
+            routes_output = utils.check_output(['route', '-n'],
                 stderr=subprocess.PIPE)
         except subprocess.CalledProcessError:
             logger.exception('Failed to get IP routes. %r' % {
@@ -415,50 +461,65 @@ class Server(Config):
             })
             raise
 
-        primary_interface = None
-        for line in routes.splitlines():
+        routes = {}
+        for line in routes_output.splitlines():
             line_split = line.split()
-            if line_split[0] == match_network:
-                primary_interface = line_split[7]
-                break
+            if len(line_split) < 8 or not re.match(IP_REGEX, line_split[0]):
+                continue
+            routes[line_split[0]] = line_split[7]
 
-        if not primary_interface and match_network != '0.0.0.0':
-            logger.debug('Failed to find interface for local network ' + \
-                    'route, using default route instead. %r' % {
+        if '0.0.0.0' not in routes:
+            logger.error('Failed to find default network interface. %r' % {
                 'server_id': self.id,
             })
-            match_network = '0.0.0.0'
-            for line in routes.splitlines():
-                line_split = line.split()
-                if line_split[0] == match_network:
-                    primary_interface = line_split[7]
-                    break
+            raise ValueError('Failed to find default network interface')
+        default_interface = routes['0.0.0.0']
 
-        args += ['-s', self.network, '-o', primary_interface,
-            '-j', 'MASQUERADE']
+        for network_address in self.local_networks or ['0.0.0.0/0']:
+            args = []
+            network = self._parse_network(network_address)[0]
 
-        return args
+            if network not in routes:
+                logger.debug('Failed to find interface for local network ' + \
+                        'route, using default route. %r' % {
+                    'server_id': self.id,
+                })
+                interface = default_interface
+            else:
+                interface = routes[network]
 
-    def _exists_iptables_rule(self):
-        try:
-            logger.debug('Checking for iptables rule. %r' % {
-                'server_id': self.id,
-            })
-            subprocess.check_call(['iptables', '-t', 'nat', '-C',
-                'POSTROUTING'] + self._generate_iptable_rule(),
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        except subprocess.CalledProcessError:
-            return False
+            if network != '0.0.0.0':
+                args += ['-d', network_address]
+
+            args += ['-s', self.network, '-o', interface, '-j', 'MASQUERADE']
+            iptable_rules.append(args)
+
+        return iptable_rules
+
+    def _exists_iptable_rules(self):
+        logger.debug('Checking for iptable rules. %r' % {
+            'server_id': self.id,
+        })
+        for iptable_rule in self._generate_iptable_rules():
+            try:
+                subprocess.check_call(['iptables', '-t', 'nat', '-C',
+                    'POSTROUTING'] + iptable_rule,
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            except subprocess.CalledProcessError:
+                return False
         return True
 
-    def _set_iptables_rule(self):
-        if not self._exists_iptables_rule():
-            logger.debug('Setting iptables rule. %r' % {
-                'server_id': self.id,
-            })
+    def _set_iptable_rules(self):
+        if self._exists_iptable_rules():
+            return
+
+        logger.debug('Setting iptable rules. %r' % {
+            'server_id': self.id,
+        })
+        for iptable_rule in self._generate_iptable_rules():
             try:
                 subprocess.check_call(['iptables', '-t', 'nat', '-A',
-                    'POSTROUTING'] + self._generate_iptable_rule(),
+                    'POSTROUTING'] + iptable_rule,
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             except subprocess.CalledProcessError:
                 logger.exception('Failed to apply iptables ' + \
@@ -468,13 +529,16 @@ class Server(Config):
                 raise
 
     def _clear_iptable_rules(self):
-        if self._exists_iptables_rule():
-            logger.debug('Clearing iptables rule. %r' % {
-                'server_id': self.id,
-            })
+        if not self._exists_iptable_rules():
+            return
+        logger.debug('Clearing iptable rules. %r' % {
+            'server_id': self.id,
+        })
+
+        for iptable_rule in self._generate_iptable_rules():
             try:
                 subprocess.check_call(['iptables', '-t', 'nat', '-D',
-                    'POSTROUTING'] + self._generate_iptable_rule(),
+                    'POSTROUTING'] + iptable_rule,
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             except subprocess.CalledProcessError:
                 logger.exception('Failed to clear iptables ' + \
@@ -483,6 +547,18 @@ class Server(Config):
                     })
                 raise
 
+    def _sub_thread(self, process):
+        for message in cache_db.subscribe(self.get_cache_key()):
+            try:
+                if message == 'stop':
+                    process.send_signal(signal.SIGINT)
+                elif message == 'force_stop':
+                    process.send_signal(signal.SIGKILL)
+                elif message == 'stopped':
+                    break
+            except OSError:
+                pass
+
     def _status_thread(self):
         i = 0
         cur_client_count = 0
@@ -490,73 +566,73 @@ class Server(Config):
             # Check interrupt every 0.1s check client count every 1s
             if i == 9:
                 i = 0
-                client_count = len(self.get_clients())
+                client_count = len(self.update_clients())
                 if client_count != cur_client_count:
                     cur_client_count = client_count
-                    Event(type=USERS_UPDATED)
+                    for org in self.iter_orgs():
+                        Event(type=USERS_UPDATED, resource_id=org.id)
                     Event(type=SERVERS_UPDATED)
             else:
                 i += 1
             time.sleep(0.1)
         self._clear_iptable_rules()
-        _events[self.id].set()
-        try:
-            del _events[self.id]
-        except KeyError:
-            pass
 
-    def _run(self):
+    def _run_thread(self):
         logger.debug('Starting ovpn process. %r' % {
             'server_id': self.id,
         })
         self._interrupt = False
         try:
-            threading.Thread(target=self._status_thread).start()
-
             try:
                 process = subprocess.Popen(['openvpn', self.ovpn_conf_path],
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                _process[self.id] = process
-                _events[self.id].set()
             except OSError:
-                _output[self.id] += traceback.format_exc()
-                self._event_delay(type=SERVER_OUTPUT_UPDATED,
-                    resource_id=self.id)
+                self.push_output(traceback.format_exc())
                 logger.exception('Failed to start ovpn process. %r' % {
                     'server_id': self.id,
                 })
+                cache_db.publish(self.get_cache_key(), 'stopped')
                 return
+            cache_db.dict_set(self.get_cache_key(), 'start_time',
+                str(int(time.time() - 1)))
+            sub_thread = threading.Thread(target=self._sub_thread,
+                args=(process,))
+            sub_thread.start()
+            status_thread = threading.Thread(target=self._status_thread)
+            status_thread.start()
+            self.status = True
+            cache_db.publish(self.get_cache_key(), 'started')
 
             while True:
                 line = process.stdout.readline()
-                if line == '' and process.poll() is not None:
-                    break
-                _output[self.id] += line
-                self._event_delay(type=SERVER_OUTPUT_UPDATED,
-                    resource_id=self.id)
+                if not line:
+                    if process.poll() is not None:
+                        break
+                    else:
+                        continue
+                self.push_output(line)
+
+            self._interrupt = True
+            status_thread.join()
+
+            self.status = False
+            cache_db.publish(self.get_cache_key(), 'stopped')
+            if process.returncode != 0:
+                Event(type=SERVERS_UPDATED)
+                LogEntry(message='Server stopped unexpectedly "%s".' % (
+                    self.name))
 
             logger.debug('Ovpn process has ended. %r' % {
                 'server_id': self.id,
             })
-        finally:
-            try:
-                del _threads[self.id]
-            except KeyError:
-                pass
-            try:
-                del _process[self.id]
-            except KeyError:
-                pass
-            try:
-                del _start_time[self.id]
-            except KeyError:
-                pass
+        except:
             self._interrupt = True
+            cache_db.publish(self.get_cache_key(), 'stopped')
 
     def start(self, silent=False):
         if self.status:
             return
-        if not self.get_orgs():
+        if not self.org_count:
             raise ValueError('Server cannot be started without ' + \
                 'any organizations')
         logger.debug('Starting server. %r' % {
@@ -564,19 +640,22 @@ class Server(Config):
         })
         self._generate_ovpn_conf()
         self._enable_ip_forwarding()
-        self._set_iptables_rule()
-        _events[self.id] = threading.Event()
-        thread = threading.Thread(target=self._run)
-        thread.start()
-        _threads[self.id] = thread
-        _start_time[self.id] = int(time.time()) - 1
-        _output[self.id] = ''
-        if not _events[self.id].wait(THREAD_EVENT_TIMEOUT):
-            raise ValueError('Server thread failed to return start event.')
-        try:
-            _events[self.id].clear()
-        except KeyError:
-            pass
+        self._set_iptable_rules()
+        self.clear_output()
+
+        threading.Thread(target=self._run_thread).start()
+
+        started = False
+        for message in cache_db.subscribe(self.get_cache_key(),
+                SUB_RESPONSE_TIMEOUT):
+            if message == 'started':
+                started = True
+                break
+            elif message == 'stopped':
+                raise ValueError('Server failed to start')
+        if not started:
+            raise ValueError('Server thread failed to return start event')
+
         if not silent:
             Event(type=SERVERS_UPDATED)
             LogEntry(message='Started server "%s".' % self.name)
@@ -587,9 +666,17 @@ class Server(Config):
         logger.debug('Stopping server. %r' % {
             'server_id': self.id,
         })
-        _process[self.id].send_signal(signal.SIGINT)
-        if not _events[self.id].wait(THREAD_EVENT_TIMEOUT):
-            raise ValueError('Server thread failed to return stop event.')
+
+        stopped = False
+        cache_db.publish(self.get_cache_key(), 'stop')
+        for message in cache_db.subscribe(self.get_cache_key(),
+                SUB_RESPONSE_TIMEOUT):
+            if message == 'stopped':
+                stopped = True
+                break
+        if not stopped:
+            raise ValueError('Server thread failed to return stop event')
+
         if not silent:
             Event(type=SERVERS_UPDATED)
             LogEntry(message='Stopped server "%s".' % self.name)
@@ -597,12 +684,29 @@ class Server(Config):
     def force_stop(self, silent=False):
         if not self.status:
             return
-        logger.info('Forcing stop server. %r' % {
+        logger.debug('Forcing stop server. %r' % {
             'server_id': self.id,
         })
-        _process[self.id].send_signal(signal.SIGKILL)
-        if not _events[self.id].wait(THREAD_EVENT_TIMEOUT):
-            raise ValueError('Server thread failed to return stop event.')
+
+        stopped = False
+        cache_db.publish(self.get_cache_key(), 'stop')
+        for message in cache_db.subscribe(self.get_cache_key(), 2):
+            if message == 'stopped':
+                stopped = True
+                break
+
+        if not stopped:
+            stopped = False
+            cache_db.publish(self.get_cache_key(), 'force_stop')
+            for message in cache_db.subscribe(self.get_cache_key(),
+                    SUB_RESPONSE_TIMEOUT):
+                if message == 'stopped':
+                    stopped = True
+                    break
+
+            if not stopped:
+                raise ValueError('Server thread failed to return stop event')
+
         if not silent:
             Event(type=SERVERS_UPDATED)
             LogEntry(message='Stopped server "%s".' % self.name)
@@ -619,29 +723,20 @@ class Server(Config):
             Event(type=SERVERS_UPDATED)
             LogEntry(message='Restarted server "%s".' % self.name)
 
-    def reload(self):
-        if not self.status:
-            self.start()
-        logger.debug('Reloading server. %r' % {
-            'server_id': self.id,
-        })
-        _process[self.id].send_signal(signal.SIGUSR1)
-        LogEntry(message='Reloaded server "%s".' % self.name)
-
     def get_output(self):
-        if self.id not in _output:
-            return ''
-        return _output[self.id]
+        return self.output
 
     def clear_output(self):
-        if self.id not in _output:
-            return ''
-        _output[self.id] = ''
+        cache_db.remove(self.get_cache_key('output'))
         self._event_delay(type=SERVER_OUTPUT_UPDATED, resource_id=self.id)
 
-    def get_clients(self):
+    def push_output(self, output):
+        cache_db.list_rpush(self.get_cache_key('output'), output.rstrip('\n'))
+        self._event_delay(type=SERVER_OUTPUT_UPDATED, resource_id=self.id)
+
+    def update_clients(self):
         if not self.status:
-            return []
+            return {}
         clients = {}
 
         if os.path.isfile(self.ovpn_status_path):
@@ -664,24 +759,34 @@ class Server(Config):
                         'connected_since': connected_since,
                     }
 
+        self.clients = clients
         return clients
 
     @staticmethod
-    def get_servers():
+    def get_server(id):
+        from node_server import NodeServer
+        if os.path.isfile(os.path.join(app_server.data_path, SERVERS_DIR,
+                id, NODE_SERVER_NAME)):
+            server = NodeServer(id=id)
+        else:
+            server = Server(id=id)
+        try:
+            server.load()
+        except IOError:
+            logger.exception('Failed to load server conf. %r' % {
+                    'server_id': id,
+                })
+            return
+        return server
+
+    @staticmethod
+    def iter_servers():
         logger.debug('Getting servers.')
-        data_path = app_server.data_path
-        path = os.path.join(data_path, SERVERS_DIR)
+        path = os.path.join(app_server.data_path, SERVERS_DIR)
         servers = []
         if os.path.isdir(path):
             for server_id in os.listdir(path):
-                server = Server(server_id)
-                try:
-                    server.load()
-                except IOError:
-                    logger.exception('Failed to load server conf, ' +
-                        'ignoring server. %r' % {
-                            'server_id': server_id,
-                        })
-                    continue
-                servers.append(server)
+                server = Server.get_server(id=server_id)
+                if server:
+                    servers.append(server)
         return servers
