@@ -1,5 +1,7 @@
 from constants import *
-from pritunl import app_server, openssl_lock
+from pritunl import app_server
+from cache import cache_db
+from cache_trie import CacheTrie
 from config import Config
 from log_entry import LogEntry
 from event import Event
@@ -10,59 +12,68 @@ import subprocess
 import logging
 import hashlib
 import base64
+import utils
+import struct
+import hmac
+import time
+import threading
 
 logger = logging.getLogger(APP_NAME)
 
 class User(Config):
     str_options = {'name', 'otp_secret', 'type'}
+    default_options = {
+        'name': 'undefined',
+    }
     chmod_mode = 0600
+    cached = True
+    cache_prefix = 'user'
 
     def __init__(self, org, id=None, name=None, type=None):
         Config.__init__(self)
         self.org = org
-        self.id = id
 
-        if type is not None:
-            self.type = type
-
-        if self.id is None:
+        if id is None:
             if type == CERT_CA:
                 self.id = CA_CERT_ID
             elif type is None:
                 raise AttributeError('Type must be specified')
             else:
                 self.id = uuid.uuid4().hex
-            self._initialized = False
+            self.name = name
+            self.type = type
         else:
-            self._initialized = True
+            self.id = id
 
         self.reqs_path = os.path.join(self.org.path, REQS_DIR,
             '%s.csr' % self.id)
-        self.ssl_conf_path = os.path.join(self.org.path, TEMP_DIR,
-            '%s.conf' % self.id)
         self.key_path = os.path.join(self.org.path, KEYS_DIR,
             '%s.key' % self.id)
         self.cert_path = os.path.join(self.org.path, CERTS_DIR,
             '%s.crt' % self.id)
-        self.key_archive_path = os.path.join(self.org.path,
-            TEMP_DIR, '%s.tar' % self.id)
+        self.temp_key_archive_path = os.path.join(self.org.path,
+            TEMP_DIR, '%s_%s.tar' % (self.id, uuid.uuid4().hex))
         self.set_path(os.path.join(self.org.path, USERS_DIR,
             '%s.conf' % self.id))
 
-        if name is not None:
-            self.name = name
+        self.temp_path = os.path.join(self.org.path, TEMP_DIR, self.id)
+        self.ssl_conf_path = os.path.join(self.temp_path, OPENSSL_NAME)
+        self.index_path = os.path.join(self.temp_path, INDEX_NAME)
+        self.index_attr_path = os.path.join(self.temp_path, INDEX_ATTR_NAME)
+        self.serial_path = os.path.join(self.temp_path, SERIAL_NAME)
 
-        if not self._initialized:
+        if id is None:
             self._initialize()
 
-    def _upgrade_0_10_2(self):
-        if not self.otp_secret:
-            logger.debug('Upgrading user to v0.10.2... %r' % {
-                'org_id': self.org.id,
-                'user_id': self.id,
-            })
-            self._generate_otp_secret()
-            self.commit()
+    def dict(self):
+        return {
+            'id': self.id,
+            'organization': self.org.id,
+            'organization_name': self.org.name,
+            'name': self.name,
+            'type': self.type,
+            'otp_secret': self.otp_secret,
+        }
 
     def _upgrade_0_10_4(self):
         if not self.type:
@@ -81,18 +92,49 @@ class User(Config):
             self.commit()
 
     def _initialize(self):
-        self._create_ssl_conf()
+        self._setup_openssl()
         self._cert_request()
         self._generate_otp_secret()
         self.commit()
         self._cert_create()
-        self._delete_ssl_conf()
+        self._clean_openssl()
+        if self.type != CERT_CA:
+            cache_db.set_add(self.org.get_cache_key('users'), self.id)
+            users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
+            users_trie.add_key(self.name, self.id)
+        self.org.sort_users_cache()
         if self.type == CERT_CLIENT:
             LogEntry(message='Created new user "%s".' % self.name)
-        Event(type=USERS_UPDATED)
+        Event(type=ORGS_UPDATED)
+        Event(type=USERS_UPDATED, resource_id=self.org.id)
+        Event(type=SERVERS_UPDATED)
+
+    def _setup_openssl(self):
+        if not os.path.exists(self.temp_path):
+            os.makedirs(self.temp_path)
+
+        with open(self.index_path, 'a'):
+            os.utime(self.index_path, None)
+
+        with open(self.index_attr_path, 'a'):
+            os.utime(self.index_attr_path, None)
+
+        with open(self.serial_path, 'w') as serial_file:
+            serial_file.write('01\n')
+
+        with open(self.ssl_conf_path, 'w') as conf_file:
+            conf_file.write(CERT_CONF % (
+                self.org.id,
+                self.org.path,
+                self.temp_path,
+                app_server.key_bits,
+                self.id,
+            ))
+
+    def _clean_openssl(self):
+        utils.rmtree(self.temp_path)
 
     def _cert_request(self):
-        openssl_lock.acquire()
         try:
             args = [
                 'openssl', 'req', '-new', '-batch',
@@ -109,12 +151,9 @@ class User(Config):
                 'user_id': self.id,
             })
             raise
-        finally:
-            openssl_lock.release()
         os.chmod(self.key_path, 0600)
 
     def _cert_create(self):
-        openssl_lock.acquire()
         try:
             args = ['openssl', 'ca', '-batch']
             if self.type == CERT_CA:
@@ -133,17 +172,6 @@ class User(Config):
                 'user_id': self.id,
             })
             raise
-        finally:
-            openssl_lock.release()
-
-    def _create_ssl_conf(self):
-        conf_data = CERT_CONF % (self.org.id, self.org.path,
-            app_server.key_bits, self.id)
-        with open(self.ssl_conf_path, 'w') as conf_file:
-            conf_file.write(conf_data)
-
-    def _delete_ssl_conf(self):
-        os.remove(self.ssl_conf_path)
 
     def _generate_otp_secret(self):
         sha_hash = hashlib.sha512()
@@ -158,54 +186,48 @@ class User(Config):
     def generate_otp_secret(self):
         self._generate_otp_secret()
         self.commit()
-        Event(type=USERS_UPDATED)
+        Event(type=USERS_UPDATED, resource_id=self.org.id)
 
-    def _revoke(self, reason):
-        if self.id == CA_CERT_ID:
-            raise TypeError('Cannot revoke ca cert')
+    def verify_otp_code(self, code):
+        otp_secret = self.otp_secret
+        padding = 8 - len(otp_secret) % 8
+        if padding != 8:
+            otp_secret = otp_secret.ljust(len(otp_secret) + padding, '=')
+        otp_secret = base64.b32decode(otp_secret.upper())
+        valid_codes = []
+        epoch = int(time.time() / 30)
+        for epoch_offset in range(-1, 2):
+            value = struct.pack('>q', epoch + epoch_offset)
+            hmac_hash = hmac.new(otp_secret, value, hashlib.sha1).digest()
+            offset = ord(hmac_hash[-1]) & 0x0F
+            truncated_hash = hmac_hash[offset:offset + 4]
+            truncated_hash = struct.unpack('>L', truncated_hash)[0]
+            truncated_hash &= 0x7FFFFFFF
+            truncated_hash %= 1000000
+            valid_codes.append('%06d' % truncated_hash)
+        if code not in valid_codes:
+            return False
 
-        if not os.path.isfile(self.cert_path):
-            logger.warning('Skipping revoke of non existent user. %r' % {
-                'org_id': self.org.id,
-                'user_id': self.id,
-            })
-            return
+        used_codes = cache_db.dict_get_all(self.get_cache_key('otp'))
+        for auth_time, used_code in used_codes.items():
+            if int(time.time()) - int(auth_time) > 120:
+                cache_db.dict_remove(self.get_cache_key('otp'), auth_time)
+            if used_code == code:
+                return False
+        cache_db.dict_set(self.get_cache_key('otp'),
+            str(int(time.time())), code)
 
-        openssl_lock.acquire()
-        try:
-            self._create_ssl_conf()
-            args = ['openssl', 'ca', '-batch',
-                '-config', self.ssl_conf_path,
-                '-revoke', self.cert_path,
-                '-crl_reason', reason
-            ]
-            proc = subprocess.Popen(args, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE)
-            returncode = proc.wait()
-            if returncode != 0:
-                err_output = proc.communicate()[1]
-                if 'ERROR:Already revoked' not in err_output:
-                    raise subprocess.CalledProcessError(returncode, args)
-            self._delete_ssl_conf()
-        except subprocess.CalledProcessError:
-            logger.exception('Failed to revoke user cert. %r' % {
-                'org_id': self.org.id,
-                'user_id': self.id,
-            })
-            raise
-        finally:
-            openssl_lock.release()
-        self.org.generate_crl()
+        return True
 
     def _build_key_archive(self):
         user_key_arcname = '%s_%s.key' % (self.org.name, self.name)
         user_cert_arcname = '%s_%s.crt' % (self.org.name, self.name)
 
-        tar_file = tarfile.open(self.key_archive_path, 'w')
+        tar_file = tarfile.open(self.temp_key_archive_path, 'w')
         try:
             tar_file.add(self.key_path, arcname=user_key_arcname)
             tar_file.add(self.cert_path, arcname=user_cert_arcname)
-            for server in self.org.get_servers():
+            for server in self.org.iter_servers():
                 server_cert_arcname = '%s_%s_%s.crt' % (
                     self.org.name, self.name, server.name)
                 server_conf_path = os.path.join(self.org.path,
@@ -233,19 +255,12 @@ class User(Config):
         finally:
             tar_file.close()
 
-        return self.key_archive_path
-
-    def _get_cert_block(self, cert_path):
-        with open(cert_path) as cert_file:
-            cert_file = cert_file.read()
-            start_index = cert_file.index('-----BEGIN CERTIFICATE-----')
-            end_index = cert_file.index('-----END CERTIFICATE-----') + 25
-            return cert_file[start_index:end_index]
+        return self.temp_key_archive_path
 
     def _build_inline_key_archive(self):
-        tar_file = tarfile.open(self.key_archive_path, 'w')
+        tar_file = tarfile.open(self.temp_key_archive_path, 'w')
         try:
-            for server in self.org.get_servers():
+            for server in self.org.iter_servers():
                 server_conf_path = os.path.join(self.org.path,
                     TEMP_DIR, '%s_%s.ovpn' % (self.id, server.id))
                 server_conf_arcname = '%s_%s_%s.ovpn' % (
@@ -260,9 +275,9 @@ class User(Config):
                 if server.otp_auth:
                     client_conf += 'auth-user-pass\n'
 
-                client_conf += '<ca>\n%s\n</ca>\n' % self._get_cert_block(
+                client_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
                     server.ca_cert_path)
-                client_conf += '<cert>\n%s\n</cert>\n' % self._get_cert_block(
+                client_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
                     self.cert_path)
                 client_conf += '<key>\n%s\n</key>\n' % open(
                     self.key_path).read().strip()
@@ -275,7 +290,7 @@ class User(Config):
         finally:
             tar_file.close()
 
-        return self.key_archive_path
+        return self.temp_key_archive_path
 
     def build_key_archive(self):
         if app_server.inline_certs:
@@ -283,6 +298,12 @@ class User(Config):
         else:
             return self._build_key_archive()
 
+    def clean_key_archive(self):
+        try:
+            os.remove(self.temp_key_archive_path)
+        except OSError:
+            pass
+
     def build_key_conf(self, server_id):
         server = self.org.get_server(server_id)
         conf_name = '%s_%s_%s.ovpn' % (self.org.name, self.name, server.name)
@@ -296,9 +317,9 @@ class User(Config):
         if server.otp_auth:
             client_conf += 'auth-user-pass\n'
 
-        client_conf += '<ca>\n%s\n</ca>\n' % self._get_cert_block(
+        client_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
             server.ca_cert_path)
-        client_conf += '<cert>\n%s\n</cert>\n' % self._get_cert_block(
+        client_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
             self.cert_path)
         client_conf += '<key>\n%s\n</key>\n' % open(
             self.key_path).read().strip()
@@ -308,15 +329,26 @@ class User(Config):
             'conf': client_conf,
         }
 
+    def clear_cache(self):
+        if self.type != CERT_CA:
+            cache_db.set_remove(self.org.get_cache_key('users'), self.id)
+            cache_db.list_remove(self.org.get_cache_key('users_sorted'),
+                self.id)
+            cache_db.decrement(self.org.get_cache_key('user_count'))
+            users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
+            users_trie.remove_key(self.name, self.id)
+        Config.clear_cache(self)
+
     def rename(self, name):
         self.name = name
         self.commit()
-        Event(type=USERS_UPDATED)
+        self.org.sort_users_cache()
+        Event(type=USERS_UPDATED, resource_id=self.org.id)
 
-    def remove(self, reason=UNSPECIFIED):
+    def remove(self):
+        self.clear_cache()
         name = self.name
         type = self.type
-        self._revoke(reason)
 
         try:
             os.remove(self.reqs_path)
@@ -328,11 +360,6 @@ class User(Config):
                 'error': error,
             })
 
-        try:
-            os.remove(self.ssl_conf_path)
-        except OSError, error:
-            pass
-
         try:
             os.remove(self.key_path)
         except OSError, error:
@@ -363,6 +390,22 @@ class User(Config):
                 'error': error,
             })
 
-        Event(type=USERS_UPDATED)
+        self._clean_openssl()
+
+        Event(type=ORGS_UPDATED)
+        Event(type=USERS_UPDATED, resource_id=self.org.id)
         if type == CERT_CLIENT:
             LogEntry(message='Deleted user "%s".' % name)
+
+    @staticmethod
+    def get_user(org, id):
+        user = User(org, id=id)
+        try:
+            user.load()
+        except IOError:
+            logger.exception('Failed to load user conf. %r' % {
+                'org_id': org.id,
+                'user_id': id,
+            })
+            return
+        return user
