@@ -1,4 +1,3 @@
-from pritunl import database
 import threading
 import unittest
 import requests
@@ -6,18 +5,18 @@ import json
 import time
 import os
 
-BASE_URL = 'http://localhost:9700'
+BASE_URL = os.getenv('BASE_URL', 'http://localhost:9700')
 HEADERS = {
     'Accept': 'application/json',
 }
-USERNAME = 'admin'
-PASSWORD = 'admin'
+USERNAME = os.getenv('USERNAME', 'admin')
+PASSWORD = os.getenv('PASSWORD', 'admin')
 TEST_PASSWORD = 'unittest'
 TEST_USER_NAME = 'unittest_user'
 TEST_ORG_NAME = 'unittest_org'
 TEST_SERVER_NAME = 'unittest_server'
-TEMP_DATABSE_PATH = 'pritunl_test.db'
 ENABLE_STANDARD_TESTS = True
+ENABLE_EXTENDED_TESTS = False
 ENABLE_STRESS_TESTS = False
 THREADED_STRESS_TEST = True
 UUID_RE = r'^[a-z0-9]+$'
@@ -31,7 +30,7 @@ AUTH_HANDLERS = [
     ('POST', '/organization'),
     ('PUT', '/organization/0'),
     ('DELETE', '/organization/0'),
-    ('POST', '/password'),
+    ('PUT', '/password'),
     ('GET', '/server'),
     ('POST', '/server'),
     ('PUT', '/server/0'),
@@ -51,23 +50,48 @@ AUTH_HANDLERS = [
 ]
 RUN_ONLY = []
 
+def _log_request(method, endpoint, start_time):
+    if endpoint.startswith('/event'):
+        return
+    response_time = int((time.time() - start_time) * 1000)
+    if endpoint.startswith('/auth'):
+        if response_time > 900:
+            color = '\033[92m'
+        elif response_time > 700:
+            color = '\033[93m'
+        else:
+            color = '\033[92m'
+    else:
+        if response_time > 400:
+            color = '\033[91m'
+        elif response_time > 200:
+            color = '\033[93m'
+        else:
+            color = '\033[92m'
+    print '%s%sms:%s:%s\033[0m' % (color, response_time, method, endpoint)
+
 _request = requests.api.request
 def request(method, endpoint, **kwargs):
     headers = {
         'Accept': 'application/json',
     }
-    if 'json' in kwargs and kwargs['json']:
+    if 'headers' in kwargs:
+        headers.update(kwargs.pop('headers'))
+    if 'json_data' in kwargs and kwargs['json_data']:
         headers['Content-Type'] = 'application/json'
-        kwargs['data'] = json.dumps(kwargs.pop('json'))
-    return _request(method, BASE_URL + endpoint, headers=headers,
+        kwargs['data'] = json.dumps(kwargs.pop('json_data'))
+    start_time = time.time()
+    response = _request(method, BASE_URL + endpoint, headers=headers,
         verify=False, **kwargs)
+    _log_request(method, endpoint, start_time)
+    return response
 requests.api.request = request
 
 
 class Session:
     def __init__(self):
         self._session = requests.Session()
-        self.response = self.post('/auth', json={
+        self.response = self.post('/auth', json_data={
             'username': USERNAME,
             'password': PASSWORD,
         })
@@ -76,11 +100,16 @@ class Session:
         headers = {
             'Accept': 'application/json',
         }
-        if 'json' in kwargs and kwargs['json']:
+        if 'headers' in kwargs:
+            headers.update(kwargs.pop('headers'))
+        if 'json_data' in kwargs and kwargs['json_data']:
             headers['Content-Type'] = 'application/json'
-            kwargs['data'] = json.dumps(kwargs.pop('json'))
-        return getattr(self._session, method)(BASE_URL + endpoint,
+            kwargs['data'] = json.dumps(kwargs.pop('json_data'))
+        start_time = time.time()
+        response = getattr(self._session, method)(BASE_URL + endpoint,
             headers=headers, verify=False, **kwargs)
+        _log_request(method, endpoint, start_time)
+        return response
 
     def get(self, endpoint, **kwargs):
         return self._request('get', endpoint, **kwargs)
@@ -95,8 +124,9 @@ class Session:
         return self._request('delete', endpoint, **kwargs)
 
 
-_global_session = Session()
 class SessionTestCase(unittest.TestCase):
+    session = Session()
+
     def setUp(self):
         if RUN_ONLY and self._testMethodName not in RUN_ONLY:
             self.skipTest('ignore')
@@ -104,8 +134,43 @@ class SessionTestCase(unittest.TestCase):
         self.org_id = None
         self.user_id = None
         self.server_id = None
-        self.session = _global_session
         self._create_test_data()
+        self._clean_test_data()
+
+    def _clean_test_data(self):
+        response = self.session.get('/organization')
+        self.assertEqual(response.status_code, 200)
+        data = response.json()
+        for org in data:
+            if org['name'] in {TEST_ORG_NAME + '2', TEST_ORG_NAME + '3'}:
+                response = self.session.delete('/organization/%s' % org['id'])
+                self.assertEqual(response.status_code, 200)
+
+        response = self.session.get('/user/%s' % self.org_id)
+        self.assertEqual(response.status_code, 200)
+        data = response.json()
+        for user in data:
+            if user['name'] == {TEST_USER_NAME + '2', TEST_USER_NAME + '3'}:
+                response = self.session.delete('/user/%s/%s' % (
+                    self.org_id, user['id']))
+                self.assertEqual(response.status_code, 200)
+
+        response = self.session.get('/server')
+        self.assertEqual(response.status_code, 200)
+        data = response.json()
+        for server in data:
+            if server['name'] == {TEST_SERVER_NAME + '2',
+                    TEST_SERVER_NAME + '3'}:
+                response = self.session.delete('/server/%s' % server['id'])
+                self.assertEqual(response.status_code, 200)
+
+        response = self.session.get('/server/%s/organization' % self.server_id)
+        self.assertEqual(response.status_code, 200)
+        data = response.json()
+        for server_org in data:
+            response = self.session.delete('/server/%s/organization/%s' % (
+                self.server_id, server_org['id']))
+            self.assertEqual(response.status_code, 200)
 
     def _create_test_data(self):
         if not self.org_id:
@@ -117,7 +182,7 @@ class SessionTestCase(unittest.TestCase):
                     self.org_id = org['id']
 
         if not self.org_id:
-            response = self.session.post('/organization', json={
+            response = self.session.post('/organization', json_data={
                 'name': TEST_ORG_NAME,
             })
             self.assertEqual(response.status_code, 200)
@@ -136,7 +201,7 @@ class SessionTestCase(unittest.TestCase):
                     self.user_id = user['id']
 
         if not self.user_id:
-            response = self.session.post('/user/%s' % self.org_id, json={
+            response = self.session.post('/user/%s' % self.org_id, json_data={
                 'name': TEST_USER_NAME,
             })
             self.assertEqual(response.status_code, 200)
@@ -155,7 +220,7 @@ class SessionTestCase(unittest.TestCase):
                     self.server_id = server['id']
 
         if not self.server_id:
-            response = self.session.post('/server', json={
+            response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME,
                 'otp_auth': True,
             })
@@ -171,45 +236,6 @@ class SessionTestCase(unittest.TestCase):
         self.assertEqual(response.status_code, 200)
 
 
-class Database(SessionTestCase):
-    def _test_db(self, db):
-        db.set('column_family', 'row1', 'column1', 'value1')
-        db.set('column_family', 'row2', 'column2', 'value2')
-        db.set('column_family', 'row3', 'column3', 'value3')
-        value = db.get('column_family', 'row1', 'column1')
-        self.assertEqual(value, 'value1')
-        value = db.get('column_family', 'row2', 'column2')
-        self.assertEqual(value, 'value2')
-        value = db.get('column_family', 'row3', 'column3')
-        self.assertEqual(value, 'value3')
-        value = db.get('column_family', 'row1')
-        self.assertEqual(value, {'column1': 'value1'})
-        value = db.get('column_family')
-        self.assertEqual(value, {
-            'row1': {'column1': 'value1'},
-            'row2': {'column2': 'value2'},
-            'row3': {'column3': 'value3'},
-        })
-        db.remove('column_family', 'row1')
-        db.remove('column_family', 'row2')
-        db.remove('column_family', 'row3')
-        value = db.get('column_family')
-        self.assertEqual(value, {})
-
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_mem(self):
-        db = database.Database(None)
-        self._test_db(db)
-        db.close()
-
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_berkeley_db(self):
-        db = database.Database(TEMP_DATABSE_PATH)
-        self._test_db(db)
-        db.close()
-        os.remove(TEMP_DATABSE_PATH)
-
-
 class Auth(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_get(self):
@@ -224,37 +250,107 @@ class Auth(SessionTestCase):
         session = Session()
         data = session.response.json()
         self.assertEqual(session.response.status_code, 200)
-        self.assertEqual(session.response.status_code, 200)
         self.assertIn('authenticated', data)
         self.assertTrue(data['authenticated'])
 
+
         response = session.get('/auth')
-        data = response.json()
         self.assertEqual(response.status_code, 200)
+
+        data = response.json()
         self.assertIn('authenticated', data)
         self.assertTrue(data['authenticated'])
 
+
         response = session.delete('/auth')
-        data = response.json()
         self.assertEqual(response.status_code, 200)
+
+        data = response.json()
         self.assertIn('authenticated', data)
         self.assertFalse(data['authenticated'])
 
+
         response = session.get('/auth')
-        data = response.json()
         self.assertEqual(response.status_code, 200)
+
+        data = response.json()
         self.assertIn('authenticated', data)
         self.assertFalse(data['authenticated'])
 
+    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
+    def test_auth_post_error(self):
+        for endpoint in ['/auth', '/auth/token']:
+            for username, password in [
+                        ('admin', 'test'),
+                        ('test', 'admin'),
+                        ('test', 'test'),
+                    ]:
+                response = requests.post(endpoint, json_data={
+                    'username': username,
+                    'password': password,
+                })
+                self.assertEqual(response.status_code, 401)
+
+                data = response.json()
+                self.assertIn('error', data)
+                self.assertEqual(data['error'], 'auth_invalid')
+                self.assertIn('error_msg', data)
+
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_auth_handlers(self):
         for method, endpoint in AUTH_HANDLERS:
             response = getattr(requests, method.lower())(endpoint)
             self.assertEqual(response.status_code, 401)
 
+    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
+    def test_auth_token_get_post_delete(self):
+        response = requests.get('/auth')
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertIn('authenticated', data)
+        self.assertFalse(data['authenticated'])
+
+
+        response = requests.post('/auth/token', json_data={
+            'username': USERNAME,
+            'password': PASSWORD,
+        })
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertIn('auth_token', data)
+        self.assertRegexpMatches(data['auth_token'], UUID_RE)
+        auth_token = data['auth_token']
+
+
+        response = requests.get('/auth', headers={
+            'Auth-Token': auth_token,
+        })
+        self.assertEqual(response.status_code, 200)
+        data = response.json()
+        self.assertIn('authenticated', data)
+        self.assertTrue(data['authenticated'])
+
+
+        response = requests.delete('/auth/token/%s' % auth_token)
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertEqual(data, {})
+
+
+        response = requests.get('/auth', headers={
+            'Auth-Token': auth_token,
+        })
+        self.assertEqual(response.status_code, 200)
+
+        data = response.json()
+        self.assertIn('authenticated', data)
+        self.assertFalse(data['authenticated'])
 
 class Data(SessionTestCase):
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
+    @unittest.skipUnless(ENABLE_EXTENDED_TESTS, 'Skipping test')
     def test_export_get(self):
         for endpoint in ['/export', '/export/pritunl.tar']:
             response = self.session.get(endpoint)
@@ -269,24 +365,6 @@ class Data(SessionTestCase):
             self.assertRegexpMatches(content_disposition, exp)
 
 
-class Event(SessionTestCase):
-    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
-    def test_event_get(self):
-        response = self.session.get('/event')
-        self.assertEqual(response.status_code, 200)
-
-        data = response.json()
-        self.assertEqual(len(data), 1)
-        self.assertIn('id', data[0])
-        self.assertIn('type', data[0])
-        self.assertIn('time', data[0])
-        self.assertIn('resource_id', data[0])
-        self.assertIsInstance(data[0]['id'], basestring)
-        self.assertEqual(data[0]['type'], 'time')
-        self.assertIsInstance(data[0]['time'], int)
-        self.assertEqual(data[0]['resource_id'], None)
-
-
 class Key(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_user_key_archive_get(self):
@@ -414,7 +492,7 @@ class Log(SessionTestCase):
 class Org(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_org_post_put_get_delete(self):
-        response = self.session.post('/organization', json={
+        response = self.session.post('/organization', json_data={
             'name': TEST_ORG_NAME + '2',
         })
         self.assertEqual(response.status_code, 200)
@@ -427,7 +505,7 @@ class Org(SessionTestCase):
         org_id = data['id']
 
 
-        response = self.session.put('/organization/%s' % org_id, json={
+        response = self.session.put('/organization/%s' % org_id, json_data={
             'name': TEST_ORG_NAME + '3',
         })
         self.assertEqual(response.status_code, 200)
@@ -472,18 +550,18 @@ class Org(SessionTestCase):
 class Password(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_password_post(self):
-        response = self.session.put('/password', json={
+        response = self.session.put('/password', json_data={
             'password': TEST_PASSWORD,
         })
         self.assertEqual(response.status_code, 200)
 
-        response = self.session.post('/auth/token', json={
+        response = self.session.post('/auth/token', json_data={
             'username': USERNAME,
             'password': TEST_PASSWORD,
         })
         self.assertEqual(response.status_code, 200)
 
-        response = self.session.put('/password', json={
+        response = self.session.put('/password', json_data={
             'password': PASSWORD,
         })
         self.assertEqual(response.status_code, 200)
@@ -492,13 +570,13 @@ class Password(SessionTestCase):
 class Server(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_server_post_put_get_delete(self):
-        response = self.session.post('/server', json={
+        response = self.session.post('/server', json_data={
             'name': TEST_SERVER_NAME + '2',
             'network': '10.254.254.0/24',
             'interface': 'tun64',
             'port': 12345,
             'protocol': 'udp',
-            'local_network': None,
+            'local_networks': [],
             'public_address': '8.8.8.8',
             'debug': True,
             'otp_auth': False,
@@ -519,8 +597,8 @@ class Server(SessionTestCase):
         self.assertEqual(data['port'], 12345)
         self.assertIn('protocol', data)
         self.assertEqual(data['protocol'], 'udp')
-        self.assertIn('local_network', data)
-        self.assertIsNone(data['local_network'])
+        self.assertIn('local_networks', data)
+        self.assertEqual(data['local_networks'], [])
         self.assertIn('public_address', data)
         self.assertEqual(data['public_address'], '8.8.8.8')
         self.assertIn('debug', data)
@@ -532,7 +610,7 @@ class Server(SessionTestCase):
         server_id = data['id']
 
 
-        response = self.session.put('/server/%s' % server_id, json={
+        response = self.session.put('/server/%s' % server_id, json_data={
             'name': TEST_SERVER_NAME + '3',
         })
         self.assertEqual(response.status_code, 200)
@@ -550,8 +628,8 @@ class Server(SessionTestCase):
         self.assertEqual(data['port'], 12345)
         self.assertIn('protocol', data)
         self.assertEqual(data['protocol'], 'udp')
-        self.assertIn('local_network', data)
-        self.assertIsNone(data['local_network'])
+        self.assertIn('local_networks', data)
+        self.assertEqual(data['local_networks'], [])
         self.assertIn('public_address', data)
         self.assertEqual(data['public_address'], '8.8.8.8')
         self.assertIn('debug', data)
@@ -576,7 +654,7 @@ class Server(SessionTestCase):
             self.assertIn('interface', server)
             self.assertIn('port', server)
             self.assertIn('protocol', server)
-            self.assertIn('local_network', server)
+            self.assertIn('local_networks', server)
             self.assertIn('public_address', server)
             self.assertIn('debug', server)
             self.assertIn('otp_auth', server)
@@ -589,7 +667,7 @@ class Server(SessionTestCase):
                 self.assertEqual(server['interface'], 'tun64')
                 self.assertEqual(server['port'], 12345)
                 self.assertEqual(server['protocol'], 'udp')
-                self.assertIsNone(server['local_network'])
+                self.assertEqual(server['local_networks'], [])
                 self.assertEqual(server['public_address'], '8.8.8.8')
                 self.assertTrue(server['debug'])
                 self.assertFalse(server['otp_auth'])
@@ -730,7 +808,7 @@ class Server(SessionTestCase):
         self.assertTrue(data['status'])
 
 
-        response = self.session.put('/server/%s' % self.server_id, json={
+        response = self.session.put('/server/%s' % self.server_id, json_data={
             'name': TEST_SERVER_NAME + '_test',
         })
         self.assertEqual(response.status_code, 400)
@@ -739,8 +817,6 @@ class Server(SessionTestCase):
         self.assertIn('error', data)
         self.assertEqual(data['error'], 'server_not_offline')
         self.assertIn('error_msg', data)
-        self.assertEqual(data['error_msg'],
-            'Server must be offline to modify settings.')
 
 
         response = self.session.put('/server/%s/organization/%s' % (
@@ -751,8 +827,6 @@ class Server(SessionTestCase):
         self.assertIn('error', data)
         self.assertEqual(data['error'], 'server_not_offline')
         self.assertIn('error_msg', data)
-        self.assertEqual(data['error_msg'], 'Server must be offline ' + \
-            'to attach an organization.')
 
 
         response = self.session.delete('/server/%s/organization/%s' % (
@@ -763,8 +837,6 @@ class Server(SessionTestCase):
         self.assertIn('error', data)
         self.assertEqual(data['error'], 'server_not_offline')
         self.assertIn('error_msg', data)
-        self.assertEqual(data['error_msg'], 'Server must be offline ' + \
-            'to detach an organization.')
 
 
         response = self.session.put('/server/%s/stop' % self.server_id)
@@ -790,7 +862,7 @@ class Server(SessionTestCase):
                     '10.255.254.1/24',
                     '10.254.254.0/24a',
                 ]:
-            response = self.session.post('/server', json={
+            response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'network': test_network,
             })
@@ -798,12 +870,8 @@ class Server(SessionTestCase):
 
             data = response.json()
             self.assertIn('error', data)
-            self.assertEqual(data['error'], 'network_not_valid')
+            self.assertEqual(data['error'], 'network_invalid')
             self.assertIn('error_msg', data)
-            self.assertEqual(data['error_msg'], 'Network address is not ' + \
-                'valid, format must be "10.[0-255].[0-255].0/[8-24]" ' + \
-                'such as "10.12.32.0/24".')
-
 
         for test_interface in [
                     'tun-1',
@@ -811,7 +879,7 @@ class Server(SessionTestCase):
                     'tun65',
                     'tuna',
                 ]:
-            response = self.session.post('/server', json={
+            response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'interface': test_interface,
             })
@@ -819,17 +887,15 @@ class Server(SessionTestCase):
 
             data = response.json()
             self.assertIn('error', data)
-            self.assertEqual(data['error'], 'interface_not_valid')
+            self.assertEqual(data['error'], 'interface_invalid')
             self.assertIn('error_msg', data)
-            self.assertEqual(data['error_msg'], 'Interface is not valid, ' + \
-                'must be "tun[0-64]" example "tun0".')
 
 
         for test_port in [
                     0,
                     65536,
                 ]:
-            response = self.session.post('/server', json={
+            response = self.session.post('/server', json_data={
                 'name': TEST_SERVER_NAME + '_test',
                 'port': test_port,
             })
@@ -837,13 +903,11 @@ class Server(SessionTestCase):
 
             data = response.json()
             self.assertIn('error', data)
-            self.assertEqual(data['error'], 'port_not_valid')
+            self.assertEqual(data['error'], 'port_invalid')
             self.assertIn('error_msg', data)
-            self.assertEqual(data['error_msg'], 'Port number is not ' + \
-                'valid, must be between 1 and 65535.')
 
 
-        response = self.session.post('/server', json={
+        response = self.session.post('/server', json_data={
             'name': TEST_SERVER_NAME + '_test',
             'protocol': 'a',
         })
@@ -851,10 +915,8 @@ class Server(SessionTestCase):
 
         data = response.json()
         self.assertIn('error', data)
-        self.assertEqual(data['error'], 'protocol_not_valid')
+        self.assertEqual(data['error'], 'protocol_invalid')
         self.assertIn('error_msg', data)
-        self.assertEqual(data['error_msg'], 'Protocol is not valid, ' + \
-            'must be "udp" or "tcp".')
 
 
 class Status(SessionTestCase):
@@ -871,12 +933,13 @@ class Status(SessionTestCase):
         self.assertIn('server_count', data)
         self.assertIn('server_version', data)
         self.assertIn('public_ip', data)
+        self.assertIn('local_networks', data)
 
 
 class User(SessionTestCase):
     @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
     def test_user_post_put_get_delete(self):
-        response = self.session.post('/user/%s' % self.org_id, json={
+        response = self.session.post('/user/%s' % self.org_id, json_data={
             'name': TEST_USER_NAME + '2',
         })
         self.assertEqual(response.status_code, 200)
@@ -896,7 +959,7 @@ class User(SessionTestCase):
 
 
         response = self.session.put('/user/%s/%s' % (self.org_id, user_id),
-            json={
+            json_data={
                 'name': TEST_USER_NAME + '3',
             })
         self.assertEqual(response.status_code, 200)
@@ -976,7 +1039,7 @@ class User(SessionTestCase):
 
 class Stress(SessionTestCase):
     def _create_user(self, org_id, name):
-        response = self.session.post('/user/%s' % org_id, json={
+        response = self.session.post('/user/%s' % org_id, json_data={
             'name': name,
         })
         self.assertEqual(response.status_code, 200)
@@ -996,7 +1059,7 @@ class Stress(SessionTestCase):
 
     @unittest.skipUnless(ENABLE_STRESS_TESTS, 'Skipping stress test')
     def test_user_post_stress(self):
-        response = self.session.post('/organization', json={
+        response = self.session.post('/organization', json_data={
             'name': TEST_ORG_NAME + '_stress',
         })
         self.assertEqual(response.status_code, 200)
@@ -1013,7 +1076,7 @@ class Stress(SessionTestCase):
             for i in xrange(8):
                 threads = []
 
-                for x in xrange(128):
+                for x in xrange(512):
                     name = '%s_%s' % (TEST_USER_NAME, str(num).zfill(4))
                     thread = threading.Thread(target=self._create_user,
                         args=(org_id, name))
@@ -1025,7 +1088,7 @@ class Stress(SessionTestCase):
                     thread.join()
 
         else:
-            for i in xrange(1024):
+            for i in xrange(4096):
                 name = '%s_%s' % (TEST_USER_NAME, str(i).zfill(4))
                 self._create_user(org_id, name)
 
