@@ -5,9 +5,9 @@ define([
   'demo/data'
 ], function($, _, Backbone, demoData) {
   'use strict';
-  /*jshint -W106:true */
+  /* jshint -W106:true */
   var routes = {};
-  var responseDelay = 150;
+  var responseDelay = 75;
 
   var uuid = function() {
     var i;
@@ -33,7 +33,7 @@ define([
     if (request.data.username !== demoData.auth.username ||
         request.data.password !== demoData.auth.password) {
       request.response({
-        error: 'auth_not_valid',
+        error: 'auth_invalid',
         error_msg: 'Username or password is not valid.'
       }, 401);
       return;
@@ -73,18 +73,32 @@ define([
     event('logs_updated');
   };
 
-  var checkEvents = function(request, lastEvent, count) {
+  var checkEvents = function(request, cursor, count) {
     setTimeout(function() {
       var i;
       var event;
       var events = [];
+      if (cursor === undefined) {
+        if (demoData.events.length) {
+          cursor = demoData.events[demoData.events.length - 1].id;
+        }
+        else {
+          cursor = null;
+        }
+      }
+      if (count === undefined) {
+        count = 0;
+      }
+      var cursorFound = cursor ? false : true;
 
       for (i = 0; i < demoData.events.length; i++) {
         event = demoData.events[i];
-        if (event.time <= lastEvent) {
-          continue;
+        if (cursorFound) {
+          events.push(event);
+        }
+        else if (event.id === cursor) {
+          cursorFound = true;
         }
-        events.push(event);
       }
 
       if (events.length) {
@@ -92,31 +106,21 @@ define([
       }
       else {
         count += 1;
-        if (count > (30 / 0.3)) {
+        if (count > (30 / 0.1)) {
           request.response([]);
         }
         else {
-          checkEvents(request, lastEvent, count);
+          checkEvents(request, cursor, count);
         }
       }
-    }, 300);
+    }, 100);
   };
 
-  var eventGet = function(request, lastEvent) {
-    if (!lastEvent) {
-      request.response([{
-        id: uuid(),
-        type: 'time',
-        resource_id: null,
-        time: Math.round(new Date().getTime() / 1000)
-      }]);
-      return;
-    }
-
-    checkEvents(request, lastEvent, 0);
+  var eventGet = function(request, cursor) {
+    checkEvents(request, cursor);
   };
   routes['GET=/event'] = eventGet;
-  routes['GET=/event/<int:lastEvent>'] = eventGet;
+  routes['GET=/event/<cursor>'] = eventGet;
 
   var logGet = function(request) {
     request.response(demoData.logs);
@@ -125,10 +129,22 @@ define([
 
   var organizationGet = function(request) {
     var id;
+    var userId;
     var orgs = [];
+    var userCount;
 
     for (id in demoData.orgs) {
-      orgs.push(demoData.orgs[id]);
+      userCount = 0;
+
+      for (userId in demoData.users[id]) {
+        if (demoData.users[id][userId].type === 'client') {
+          userCount += 1;
+        }
+      }
+
+      orgs.push(_.extend({
+        user_count: userCount
+      }, demoData.orgs[id]));
     }
 
     request.response(orgs);
@@ -213,15 +229,19 @@ define([
     demoData.servers[serverId] = {
       id: serverId,
       name: request.data.name,
+      type: request.data.type,
       network: request.data.network,
       interface: request.data.interface,
       port: request.data.port,
       protocol: request.data.protocol,
-      local_network: request.data.local_network,
+      local_networks: request.data.local_networks,
       public_address: request.data.public_address,
       otp_auth: request.data.otp_auth,
       lzo_compression: request.data.lzo_compression,
       debug: request.data.debug,
+      node_ip: request.data.node_ip,
+      node_port: request.data.node_port,
+      node_key: request.data.node_key,
       status: false,
       orgs: [],
       output: ''
@@ -238,11 +258,14 @@ define([
     demoData.servers[serverId].interface = request.data.interface;
     demoData.servers[serverId].port = request.data.port;
     demoData.servers[serverId].protocol = request.data.protocol;
-    demoData.servers[serverId].local_network = request.data.local_network;
+    demoData.servers[serverId].local_networks = request.data.local_networks;
     demoData.servers[serverId].public_address = request.data.public_address;
     demoData.servers[serverId].otp_auth = request.data.otp_auth;
     demoData.servers[serverId].lzo_compression = request.data.lzo_compression;
     demoData.servers[serverId].debug = request.data.debug;
+    demoData.servers[serverId].node_ip = request.data.node_ip;
+    demoData.servers[serverId].node_port = request.data.node_port;
+    demoData.servers[serverId].node_key = request.data.node_key;
     event('servers_updated');
     request.response({});
   };
@@ -362,7 +385,13 @@ define([
       servers_online: serversOnlineCount,
       server_count: serversCount,
       server_version: null,
-      public_ip: '8.8.8.8'
+      public_ip: '8.8.8.8',
+      local_networks: [
+        '10.10.0.0/24',
+        '10.20.1.0/24',
+        '10.30.2.0/24',
+        '192.168.1.0/24'
+      ]
     });
   };
   routes['GET=/status'] = statusGet;
@@ -376,14 +405,35 @@ define([
   var userGet = function(request, orgId) {
     var id;
     var users = [];
+    var start = (new Date).getTime();
 
     for (id in demoData.users[orgId]) {
+      if (request.data.search &&
+          demoData.users[orgId][id].name.indexOf(request.data.search) !== 0) {
+        continue;
+      }
       users.push(_.extend({
         organization_name: demoData.orgs[orgId].name
       }, demoData.users[orgId][id]));
     }
 
-    request.response(users);
+    if (request.data.search) {
+      request.response({
+          search: request.data.search,
+          search_more: false,
+          search_limit: request.data.limit,
+          search_count: users.length,
+          search_time: ((new Date).getTime() - start + 10) / 1000,
+          users: users
+      });
+    }
+    else {
+      request.response({
+          page: 0,
+          page_total: 0,
+          users: users
+      });
+    }
   };
   routes['GET=/user/<orgId>'] = userGet;
 
@@ -401,7 +451,7 @@ define([
       servers: []
     };
 
-    event('users_updated');
+    event('users_updated', orgId);
     logEntry('Created new user "' + request.data.name + '".');
     request.response({});
   };
@@ -409,7 +459,7 @@ define([
 
   var userPut = function(request, orgId, userId) {
     demoData.users[orgId][userId].name = request.data.name;
-    event('users_updated');
+    event('users_updated', orgId);
     request.response({});
   };
   routes['PUT=/user/<orgId>/<userId>'] = userPut;
@@ -417,7 +467,7 @@ define([
   var userDelete = function(request, orgId, userId) {
     var name = demoData.users[orgId][userId].name;
     delete demoData.users[orgId][userId];
-    event('users_updated');
+    event('users_updated', orgId);
     logEntry('Deleted user "' + name + '".');
     request.response({});
   };
@@ -425,7 +475,7 @@ define([
 
   var userOtpSecretPut = function(request, orgId, userId) {
     demoData.users[orgId][userId].otp_secret = secretKey();
-    event('users_updated');
+    event('users_updated', orgId);
     request.response({});
   };
   routes['PUT=/user/<orgId>/<userId>/otp_secret'] = userOtpSecretPut;
@@ -497,7 +547,8 @@ define([
       }
     }
 
-    if (ajaxRequest.data && ajaxRequest.dataType === 'json') {
+    if (ajaxRequest.data && ajaxRequest.dataType === 'json' &&
+        typeof ajaxRequest.data === 'string') {
       ajaxRequest.data = JSON.parse(ajaxRequest.data);
     }
 
