@@ -7,9 +7,8 @@ define([
   var KeyModel = Backbone.Model.extend({
     defaults: {
       'id': null,
-      'organization': null,
-      'user': null,
-      'url': null
+      'key_url': null,
+      'view_url': null
     },
     url: function() {
       return '/key/' + this.get('organization') + '/' + this.get('user');
