@@ -8,6 +8,7 @@ define([
     defaults: {
       'id': null,
       'name': null,
+      'type': null,
       'status': null,
       'uptime': null,
       'users_online': null,
@@ -16,8 +17,11 @@ define([
       'interface': null,
       'port': null,
       'protocol': null,
-      'local_network': null,
+      'local_networks': null,
       'public_address': null,
+      'node_ip': null,
+      'node_port': null,
+      'node_key': null,
       'otp_auth': null,
       'lzo_compression': null,
       'debug': null
