@@ -14,14 +14,16 @@ define([
     initialize: function(options) {
       this.model = new ServerModel({
         'name': '',
+        'type': options.type,
         'network': '10.' + this._rand(15, 250) + '.' +
           this._rand(15, 250) + '.0/24',
         'interface': this._get_free_interface(),
         'port': this._rand(10000, 19999),
         'protocol': 'udp',
-        'public_address': options.publicIp || ''
+        'public_address': options.publicIp || '',
+        'local_networks': []
       });
-      ModalAddServerView.__super__.initialize.call(this);
+      ModalAddServerView.__super__.initialize.call(this, options);
     },
     _rand: function(min, max) {
       return Math.floor(Math.random() * (max - min + 1)) + min;
