@@ -25,8 +25,10 @@ define([
       this.model.fetch({
         success: function() {
           this.clearLoading();
-          this.$('input').val(window.location.protocol + '//' +
-            window.location.host + this.model.get('url'));
+          this.$('.key-link input').val(window.location.protocol + '//' +
+            window.location.host + this.model.get('key_url'));
+          this.$('.otp-link input').val(window.location.protocol + '//' +
+            window.location.host + this.model.get('view_url'));
         }.bind(this),
         error: function() {
           this.clearLoading();
@@ -34,8 +36,8 @@ define([
         }.bind(this)
       });
     },
-    onClickInput: function() {
-      this.$('input').select();
+    onClickInput: function(evt) {
+      this.$(evt.target).select();
     }
   });
 
