@@ -2,9 +2,10 @@ define([
   'jquery',
   'underscore',
   'backbone',
+  'select',
   'views/modal',
   'text!templates/modalServerSettings.html'
-], function($, _, Backbone, ModalView, modalServerSettingsTemplate) {
+], function($, _, Backbone, Select, ModalView, modalServerSettingsTemplate) {
   'use strict';
   var ModalServerSettingsView = ModalView.extend({
     className: 'server-settings-modal',
@@ -15,31 +16,62 @@ define([
     errorMsg: 'Failed to saving server, server error occurred.',
     events: function() {
       return _.extend({
-        'click .local-network-toggle .selector': 'onLocalNetworkSelect',
+        'change .server-mode select': 'onServerMode',
         'click .otp-auth-toggle .selector': 'onOtpAuthSelect',
         'click .debug-toggle .selector': 'onDebugSelect'
       }, ModalServerSettingsView.__super__.events);
     },
+    initialize: function(options) {
+      this.localNetworks = options.localNetworks;
+      ModalServerSettingsView.__super__.initialize.call(this);
+    },
     body: function() {
       return this.template(this.model.toJSON());
     },
-    getLocalNetworkSelect: function() {
-      return this.$('.local-network-toggle .selector').hasClass('selected');
+    postRender: function() {
+      if (this.model.get('local_networks').length) {
+        this.$('.otp-auth-toggle').appendTo('.left');
+      }
+      this.$('.local-network input').select2({
+        tags: this.localNetworks,
+        tokenSeparators: [',', ' '],
+        placeholder: 'Select Local Networks',
+        formatNoMatches: function() {
+          return 'Enter Network Address';
+        },
+        width: '200px'
+      });
+      this.$('.local-network input').select2(
+        'val', this.model.get('local_networks'));
     },
-    setLocalNetworkSelect: function(state) {
-      if (state) {
-        this.$('.local-network-toggle .selector').addClass('selected');
-        this.$('.local-network-toggle .selector-inner').show();
+    getServerMode: function() {
+      return this.$('.server-mode select').val();
+    },
+    setServerMode: function(mode) {
+      if (mode === 'lan') {
+        this.$('.server-mode select').val('lan');
         this.$('.local-network').slideDown(250);
+        this.$('.otp-auth-toggle').slideUp(250, function() {
+          this.$('.otp-auth-toggle').appendTo('.left');
+          this.$('.otp-auth-toggle').show();
+        }.bind(this));
       }
       else {
-        this.$('.local-network-toggle .selector').removeClass('selected');
-        this.$('.local-network-toggle .selector-inner').hide();
+        this.$('.server-mode select').val('all');
         this.$('.local-network').slideUp(250);
+        this.$('.otp-auth-toggle').slideUp(250, function() {
+          this.$('.otp-auth-toggle').appendTo('.right');
+          this.$('.otp-auth-toggle').show();
+        }.bind(this));
       }
     },
-    onLocalNetworkSelect: function() {
-      this.setLocalNetworkSelect(!this.getLocalNetworkSelect());
+    onServerMode: function() {
+      if (this.getServerMode() === 'lan') {
+        this.setServerMode('lan');
+      }
+      else {
+        this.setServerMode('all');
+      }
     },
     getOtpAuthSelect: function() {
       return this.$('.otp-auth-toggle .selector').hasClass('selected');
@@ -80,7 +112,14 @@ define([
       var port = this.$('.port input').val();
       var protocol = this.$('.protocol select').val();
       var publicAddress = this.$('.public-address input').val();
-      var localNetwork = null;
+      var nodeHost = this.$('.node-host input').val().split(':');
+      var nodeIP = nodeHost[0];
+      var nodePort = null;
+      if (nodeHost.length > 1) {
+        nodePort = parseInt(nodeHost[1], 10);
+      }
+      var nodeKey = this.$('.node-key input').val();
+      var localNetworks = [];
       var debug = this.getDebugSelect();
       var otpAuth = this.getOtpAuthSelect();
 
@@ -105,26 +144,45 @@ define([
           '.public-address');
         return;
       }
-      if (this.getLocalNetworkSelect()) {
-        localNetwork = this.$('.local-network input').val();
-        if (!localNetwork) {
+      if (this.getServerMode() === 'lan') {
+        localNetworks = this.$('.local-network input').select2('val');
+        if (!localNetworks) {
           this.setAlert('danger', 'Local network can not be empty.',
             '.local-network');
           return;
         }
       }
-      this.setLoading(this.loadingMsg);
-      this.model.save({
+
+      var data = {
         'name': name,
+        'type': this.model.get('type'),
         'network': network,
         'interface': iface,
         'port': port,
         'protocol': protocol,
-        'local_network': localNetwork,
+        'local_networks': localNetworks,
         'public_address': publicAddress,
         'otp_auth': otpAuth,
         'debug': debug
-      }, {
+      };
+
+      if (this.model.get('type') === 'node_server') {
+        if (!nodeIP) {
+          this.setAlert('danger', 'Node host cannot be empty.', '.node-host');
+          return;
+        }
+        if (!nodeKey) {
+          this.setAlert('danger', 'Node api key cannot be empty.',
+            '.node-key');
+          return;
+        }
+        data.node_ip = nodeIP;
+        data.node_port = nodePort;
+        data.node_key = nodeKey;
+      }
+
+      this.setLoading(this.loadingMsg);
+      this.model.save(data, {
         success: function() {
           this.close(true);
         }.bind(this),
