@@ -1,4 +1,4 @@
-/*jshint -W098:true */
+/* jshint -W098:true */
 define([
   'jquery',
   'underscore',
@@ -31,6 +31,9 @@ define([
     update: function() {
     },
     setData: function(data) {
+      if (data && data.slice(-1) !== '\n') {
+        data += '\n';
+      }
       this.editor.setValue(data);
       this.editor.navigateFileEnd();
     },
